@@ -282,20 +282,20 @@ const CDSCMEDIA* KDSC::page_media() const
     return _cdsc->page_media;
 }
 
-auto_ptr<KDSCBBOX> KDSC::bbox() const
+std::unique_ptr<KDSCBBOX> KDSC::bbox() const
 {
     if( _cdsc->bbox == nullptr )
-	return auto_ptr<KDSCBBOX>( nullptr );
+	return nullptr;
     else
-	return auto_ptr<KDSCBBOX>( new KDSCBBOX( *_cdsc->bbox ) );
+	return std::make_unique<KDSCBBOX>( *_cdsc->bbox );
 }
 
-auto_ptr<KDSCBBOX> KDSC::page_bbox() const
+std::unique_ptr<KDSCBBOX> KDSC::page_bbox() const
 {
     if( _cdsc->page_bbox == nullptr )
-	return auto_ptr<KDSCBBOX>( nullptr );
+	return nullptr;
     else
-	return auto_ptr<KDSCBBOX>( new KDSCBBOX( *_cdsc->page_bbox ) );
+	return std::make_unique<KDSCBBOX>( *_cdsc->page_bbox );
 }
 
 QString KDSC::dsc_title() const
